#property strict

#include "Indicator.mqh"

class SIMPLE_ZIGZAG : public INDICATOR
{
private:
  int MinPips;
  int Pause;

public:
  void Init( const int iMinPips = 0, const double dPause = 0 )
  {
    this.MinPips = iMinPips;
    this.Pause = (int)dPause;

    return;
  }

  SIMPLE_ZIGZAG( const int iMinPips = 0, const double dPause = 0, const bool AddEvent = FALSE  ) : INDICATOR(AddEvent)
  {
    this.Init(iMinPips, dPause);

    const string Names[] = {"Simple_ZigZag"};
    const int Types[] = {DRAW_SECTION};
    const color Colors[] = {clrYellow};

    const int Size = ::ArraySize(Names);

    ::ArrayResize(this.Buffers, Size);

    for (int i = 0; i < Size; i++)
      this.Buffers[i] = new BUFFER(Types[i], EMPTY, EMPTY, Colors[i], Names[i], TRUE, TRUE); //       -   

    ::IndicatorSetInteger(INDICATOR_DIGITS, ::_Digits);

    ::EventSetMillisecondTimer(100); //     (  )
  }

  ~SIMPLE_ZIGZAG( void )
  {
    ::EventKillTimer();
  }

  virtual int Calculate( const int rates_total,
                         const int prev_calculated,
                         const datetime &time[],
                         const double &open[],
                         const double &high[],
                         const double &low[],
                         const double &close[],
                         const long &tick_volume[],
                         const long &volume[],
                         const int &spread[] )
  {
    static bool FlagUp;
    static double MinMax;
    static int N;

    const double dMinPips = this.MinPips * ::_Point;

    if (prev_calculated == 0)
    {
      FlagUp = TRUE;
      MinMax = DBL_MIN;

    // MT4    ,    
      this.SetEmpty();
    }

    for (int i = rates_total - prev_calculated - 1; i >= 0; i--)
    {
      const double PriceHigh = high[i];
      const double PriceLow = low[i];

      if (FlagUp)
      {
        if (PriceHigh > MinMax)
        {
          MinMax = PriceHigh;
          N = i;
        }
        else if (MinMax - PriceLow >= dMinPips)
        {
          this.Buffers[0].Buffer[N] = MinMax;

          MinMax = PriceLow;
          N = i;

          FlagUp = FALSE;
        }
      }
      else
      {
        if (PriceLow < MinMax)
        {
          MinMax = PriceLow;
          N = i;
        }
        else if (PriceHigh - MinMax >= dMinPips)
        {
          this.Buffers[0].Buffer[N] = MinMax;

          MinMax = PriceHigh;
          N = i;

          FlagUp = TRUE;
        }
      }
    }

    this.Buffers[0].Buffer[N] = MinMax;

    //  ,    
    if (prev_calculated == 0)
      this.ToVisual(this.Pause);
    else
      this.ToVisual();

    return(rates_total);
  }
};